/*
 * SO_PEERCRED example
 * (c) 2009 marc welz 
 *
 * you may need to do a "grep /usr/include for ucred" to find the correct headers 
 *
 * to test run "lspc /tmp/.X11-unix/X0" if your first X11 display is up
 */

#include <stdio.h>
#include <string.h>
#include <sysexits.h>
#include <errno.h>

#include <sys/socket.h>
#include <sys/un.h>

int main(int argc, char **argv)
{
  int fd, len;
  struct sockaddr_un su;
  struct ucred cr;

  if(argc <= 1){
    fprintf(stderr, "usage: %s stream-unix-socket\n", argv[0]);
    return EX_USAGE;
  }
  
  fd = socket(AF_UNIX, SOCK_STREAM, 0);
  if(fd < 0){
    return EX_OSERR;
  }

  su.sun_family = AF_UNIX;
  if(strlen(argv[1]) > sizeof(su.sun_path)){
    fprintf(stderr, "error: path %s too long\n", argv[1]);
    return EX_OSERR; /* could also be considered a usage problem */
  }
  
  strncpy(su.sun_path, argv[1], sizeof(su.sun_path));

  if(connect(fd, (struct sockaddr *)&su, sizeof(su))){
    switch(errno){
      case ECONNREFUSED :
      case ENOENT :
        fprintf(stderr, "warning: nothing available on %s\n", argv[1]);
        return 1;
      default :
        fprintf(stderr, "error: unable to connect to %s: %s\n", argv[1], strerror(errno));
        return EX_OSERR;
    }
  }

  len = sizeof(struct ucred);
  if(getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cr, &len) < 0){
    fprintf(stderr, "error: peercred failed on %s: %s\n", argv[1], strerror(errno));
    return EX_OSERR;
  }

  /* TODO resolve name or map to proc entry */
  printf("%s: %u.%u %u\n", argv[1], cr.uid, cr.gid, cr.pid);

  return 0;
}
